from pathlib import Path


def get_folder_size(folder):
    return ByteSize(sum(file.stat().st_size for file in Path(folder).rglob('*')))


class ByteSize(int):

    _kB = 1024
    _suffixes = 'B', 'kB', 'MB', 'GB', 'PB'

    def __new__(cls, *args, **kwargs):
        return super().__new__(cls, *args, **kwargs)

    def __init__(self, *args, **kwargs):
        self.bytes = self.B = int(self)
        self.kilobytes = self.kB = self / self._kB**1
        self.megabytes = self.MB = self / self._kB**2
        self.gigabytes = self.GB = self / self._kB**3
        self.petabytes = self.PB = self / self._kB**4
        *suffixes, last = self._suffixes
        suffix = next((
            suffix
            for suffix in suffixes
            if 1 < getattr(self, suffix) < self._kB
        ), last)
        self.readable = suffix, getattr(self, suffix)

        super().__init__()

    def __str__(self):
        return self.__format__('.2f')

    def __repr__(self):
        return '{}({})'.format(self.__class__.__name__, super().__repr__())

    def __format__(self, format_spec):
        suffix, val = self.readable
        return '{val:{fmt}} {suf}'.format(val=val, fmt=format_spec, suf=suffix)

    def __sub__(self, other):
        return self.__class__(super().__sub__(other))

    def __add__(self, other):
        return self.__class__(super().__add__(other))
    
    def __mul__(self, other):
        return self.__class__(super().__mul__(other))

    def __rsub__(self, other):
        return self.__class__(super().__sub__(other))

    def __radd__(self, other):
        return self.__class__(super().__add__(other))
    
    def __rmul__(self, other):
        return self.__class__(super().__rmul__(other))   


if __name__ == "__main__":

    size = get_folder_size("myOutputFolder")
    # print(size)
    # print(size.GB)
    # print(size.gigabytes)
    # print(size.PB)
    # print(size.MB)
    print(size.B)



import ssl
import os
import time
import math
import imghdr
import requests
import numpy as np
import datetime as dt
from tqdm import tqdm
import multiprocessing
from typing import Tuple
from selenium import webdriver
import chromedriver_autoinstaller
from multiprocessing.pool import ThreadPool
from os.path import join, exists, normpath, relpath
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager






# Ignore SSL certificate errors
ctx = ssl.create_default_context()
ctx.check_hostname = False
ctx.verify_mode = ssl.CERT_NONE

headers = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/20100101 Firefox/21.0'
}





# To synchronize

LOCK_VAR = 0
UNLOCK_VAR = 0
LOCKING_LIMIT = 50 # MAX NO OF THREADS


class map_api:
    """
    Pull tiles from Apple Maps
    """
    def __init__(self, min_lat_deg:float, max_lat_deg:float, min_lon_deg:float, max_lon_deg:float, zoom= 19, ac_key:str= None, verbose:bool= False, threads_:int= 4, container_dir:str= ""):
        """
        Parameters
        -------------------------------------

        min_lat_deg: float
        max_lat_deg: float
        min_lon_deg: float
        max_lon_deg: float

        zoom: int
            Zoom level. Between  1 and 20.

        ac_key:str (default= None)
            Access key to Apple Maps. If not provided, will use a headless Chrome instance to fetch a session key.

        verbose:bool (default= False)
            Helpful debugging output

        threads_: int (default= 4)
            Thread limit for process. Max depending on CPU cores

        container_dir:str (default= "")
            When downloading images, place them in this directory.
            It will be created if it does not exist.
        """
        global LOCKING_LIMIT
        self._acKey = None
        self._containerDir = ""
        if ac_key is None:
            self._getAPIKey()
        else:
            self.ac_key = ac_key
        self.set_bounds(min_lat_deg, max_lat_deg, min_lon_deg, max_lon_deg)
        self.zoom = zoom
        self.verbose = bool(verbose)
        
        # To get the maximum number of threads
        MAX_CORES = multiprocessing.cpu_count()
        if threads_> MAX_CORES:
            print("Sorry, {} -- threads unavailable, using maximum CPU threads : {}".format(threads_,MAX_CORES))
            threads_ = MAX_CORES
        
        LOCKING_LIMIT = threads_

        if self.verbose:
            print(self.ac_key,self.min_lat_deg,self.max_lat_deg,self.min_lon_deg,self.max_lon_deg,self.zoom,self.verbose,LOCKING_LIMIT)
        self._getMasks = True
        self.container_dir = container_dir
        print("Initializing satellitescraper ... Please wait...")
        

    @property
    def container_dir(self) -> str:
        """
        Get the output directory
        """
        return self._containerDir

    @container_dir.setter
    def container_dir(self, newDir:str):
        try:
            if isinstance(newDir, str) and len(newDir) > 0:
                newDir = normpath(relpath(newDir))
                if not exists(newDir):
                    if self.verbose:
                        print(f"Creating target directory `{newDir}`")
                    os.makedirs(newDir)
                assert exists(newDir)
                self._containerDir = newDir
        except Exception: #pylint: disable= broad-except
            self._containerDir = ""


    def set_bounds(self, min_lat_deg:float, max_lat_deg:float, min_lon_deg:float, max_lon_deg:float):
        """
        Set the viewport bounds
        """
        assert -90 < min_lat_deg < 90
        assert -90 < max_lat_deg < 90
        assert min_lat_deg < max_lat_deg
        assert -180 < min_lon_deg < 180
        assert -180 < max_lon_deg < 180
        assert min_lon_deg < max_lon_deg
        # For compatibility with other funcs,
        # explicitly cast to float
        self.min_lat_deg = float(min_lat_deg)
        self.max_lat_deg = float(max_lat_deg)
        self.min_lon_deg = float(min_lon_deg)
        self.max_lon_deg = float(max_lon_deg)

    def _getLatBounds(self) -> Tuple[float, float]:
        """
        Internal getter for (min, max) latitude
        """
        return self.min_lat_deg, self.max_lat_deg

    def _getLonBounds(self) -> Tuple[float, float]:
        """
        Internal getter for (min, max) longitude
        """
        return self.min_lon_deg, self.max_lon_deg

    @property
    def ac_key(self) -> str:
        """
        Get or set the internal accessKey for the tile requests
        """
        return self._acKey

    @ac_key.setter
    def ac_key(self, newACKey:str):
        try:
            if not newACKey.startswith("&"):
                if not newACKey.lower().startswith("a"):
                    newACKey = f"accessKey={newACKey}"
                newACKey = f"&{newACKey}"
            self._acKey = newACKey
        except (AttributeError, TypeError):
            self._acKey = None
            if newACKey is not None:
                raise ValueError("Invalid AccessKey string")

    def _getAPIKey(self, timeout:float= 60) -> str:
        """
        Use a headless Chrome/Chromium instance to scrape the access key
        from the data-map-printing-background attribute of Apple Maps.
        """
        SAMPLE_KEY = r"1614125879_3642792122889215637_%2F_RwvhYZM5fKknqTdkXih2Wcu3s2f3Xea126uoIuDzUIY%3D"
        KEY_START = r"&accessKey="
        chromeDriverPath = chromedriver_autoinstaller.install(cwd= True)

        service = Service(executable_path=chromeDriverPath)
        options = webdriver.ChromeOptions()
        options.add_argument('headless')
        driver = webdriver.Chrome(service=service, options=options)
        driver.get("https://satellites.pro/USA_map#37.405074,-94.284668,5")
        keyContents = None
        loopStartTime = dt.datetime.now()
        while keyContents is None and (dt.datetime.now() - loopStartTime).total_seconds() < timeout:
            time.sleep(2.5)
            try:
                baseMap = driver.find_element(By.ID, "_MutantContainer_22")
                mapData = baseMap.get_attribute("data-map-printing-background")
                accessKeyStart = mapData.find(KEY_START)
                accessKeyEnd = accessKeyStart + int(1.5 * len(SAMPLE_KEY))
                searchForKey = mapData[accessKeyStart:accessKeyEnd]
                keyContents = searchForKey[len(KEY_START):]
                keyEnd = keyContents.find("&")
                keyContents = keyContents[:keyEnd]
            except Exception: #pylint: disable= broad-except
                keyContents = None
        if keyContents is None:
            raise TimeoutError(f"Unable to automatically fetch API key in {timeout}s")
        self.ac_key = keyContents
        return keyContents


    def ret_xy_tiles(self, lat_deg:float, lon_deg:float) -> Tuple[int, int]:
        """
        Parameters
        -----------------------

        lat_deg:float
        lon_deg:float

        Returns
        ----------------------
        tuple: (xTile, yTile)
        """
        n = 2**self.zoom
        xTile = n * ((lon_deg + 180) / 360)
        lat_rad = lat_deg * math.pi / 180.0
        yTile = n * (1 - (math.log(math.tan(lat_rad) + 1/math.cos(lat_rad)) / math.pi)) / 2
        return int(xTile),int(yTile)

    def ret_lat_lon(self, xTile:int, yTile:int) -> Tuple[float, float]:
        """
        Parameters
        -----------------------

        xTile:int
        yTile:int

        Returns
        ----------------------
        tuple: (lat, lng)
        """
        n = 2**self.zoom
        lon_deg = int(xTile)/n * 360.0 - 180.0
#         lat_rad = math.atan(math.asinh(math.pi * (1 - 2 * int(yTile)/n)))
        lat_rad=2*((math.pi/4)-math.atan(math.exp(-1*math.pi*(1-2* int(yTile)/n))))
        lat_deg = lat_rad * 180.0 / math.pi
        return lat_deg, lon_deg

    def make_url(self, lat_deg:float, lon_deg:float):
        """
        returns the list of urls when lat, lon, zoom and accessKey is provided

        Parameters
        -----------------------

        lat_deg:float
        lon_deg:float
        """
        xTile, yTile = self.ret_xy_tiles(lat_deg, lon_deg)
        return [xTile, yTile]

    def get_img(self, url_str:str, vNumber:int= 9751, getMask:bool= None, prefix:str= "", _rerun:bool= False):
        """
        Get images from the URL provided and save them

        Parameters
        --------------------
        url_str:str
            The URL to read

        vNumber:int (default= 9042)
            The original version of this number was hardcoded as 7072,
            which was no longer working. Moved to a kwarg.

        getMask:bool (default= None)
            By default, uses the internal self._getMasks variable set
            on instantiation. If set to a boolean value, overrides the
            current self._getMasks value

        _rerun:bool (default= False)
            Internal. Tracks retry status.
        """
        global headers, LOCK_VAR, UNLOCK_VAR, LOCKING_LIMIT
        if isinstance(getMask, bool):
            self._getMasks = getMask
        getMask = self._getMasks
        if self.verbose:
            print(url_str)
        UNLOCK_VAR = UNLOCK_VAR + 1
        LOCK_VAR = 1
        if self.verbose:
            print("UNLOCK VAR : ",UNLOCK_VAR)
        if UNLOCK_VAR >= LOCKING_LIMIT:
            LOCK_VAR = 0
            UNLOCK_VAR = 0
            if self.verbose:
                print("-------- UNLOCKING")
        xTile = url_str[0]
        yTile = url_str[1]
        file_name = join(self.container_dir, f"{prefix}{xTile}_{yTile}.jpg")
        try:
            assert exists(file_name)
        except AssertionError:
            try:
                # get the image tile and the mask tile for the same
                # sample sat tile: https://sat-cdn2.apple-mapkit.com/tile?style=7&size=1&scale=1&z=19&x=390843&y=228270&v=9262&accessKey=1649243787_2102081627305478489_%2F_hIz9LjsZkMj6NE7y%2BimXS9vFQbxfjLBClZR7yqyFtsE%3D&emphasis=standard&tint=light
                req_url = f"https://sat-cdn2.apple-mapkit.com/tile?style=7&size=1&scale=1&z={self.zoom}&x={xTile}&y={yTile}&v={vNumber}{self.ac_key}"
                if self.verbose:
                    print(req_url)
                r = requests.get(req_url, headers= headers)
                
                try:
                    if "access denied" in str(r.content).lower():
                        if _rerun:
                            return False
                        # Refresh the API key
                        self._getAPIKey()
                        return self.get_img(url_str, vNumber, getMask, _rerun= True)
                except Exception: #pylint: disable= broad-except
                    pass
                with open(file_name, 'wb') as fh:
                    fh.write(r.content)
                if imghdr.what(file_name) == 'jpeg':
                    if self.verbose:
                        print(file_name,"JPEG")
                else:
                    os.remove(file_name)
                    if self.verbose:
                        print(file_name, "NOT JPEG")
            except Exception as e: #pylint: disable= broad-except
                if self.verbose:
                    print(e)
        if getMask:
            ext = file_name.split('.').pop()
            file_name_road = file_name[:-len(ext)-1]+"_road.png"
            try:
                assert exists(file_name_road)
            except AssertionError:
                for cdnLevel in range(1, 5):
                    # change the 2nd auth according to the datetime for downloading the roads masks
                    today_date = str(dt.date.today())
                    year = today_date[0:4]
                    month = today_date[5:7]
                    day = today_date[8:10]
                    env_key = str(year)+str(month)+str(day)
                    # sample road tile 1: https://cdn3.apple-mapkit.com/ti/tile?country=IN&region=IN&style=46&size=1&x=390842&y=228268&z=19&scale=1&lang=en&v=2204054&poi=1&accessKey=1649243787_2102081627305478489_%2F_hIz9LjsZkMj6NE7y%2BimXS9vFQbxfjLBClZR7yqyFtsE%3D&emphasis=standard&tint=light
                    # sample road tile 2: https://cdn4.apple-mapkit.com/ti/tile?country=IN&region=IN&style=46&size=1&x=296223&y=176608&z=19&scale=1&lang=en&v=2204054&poi=1&accessKey=1649243787_2102081627305478489_%2F_hIz9LjsZkMj6NE7y%2BimXS9vFQbxfjLBClZR7yqyFtsE%3D&emphasis=standard&tint=light
                    
                    req_url = f"https://cdn{cdnLevel}.apple-mapkit.com/ti/tile?country=US&region=US&style=46&size=1&x={xTile}&y={yTile}&z={self.zoom}&scale=1&lang=en&v={env_key}4&poi=1{self.ac_key}&emphasis=standard&tint=light"
                    try:
                        # image and mask retrieval
                        # For the roads data
                        if self.verbose:
                            print(req_url)
                        r = requests.get(req_url, headers= headers)
                        with open(file_name_road, 'wb') as fh:
                            fh.write(r.content)
                        if imghdr.what(file_name_road) == 'png':
                            if self.verbose:
                                print(file_name_road,"PNG")
                            break # Success
                        else:
                            os.remove(file_name_road)
                            if self.verbose:
                                print(file_name_road,"NOT PNG")
                    except Exception as e: #pylint: disable= broad-except
                        if self.verbose:
                            print(e)

    def download(self, getMasks:bool= False, latLonResolution:float= 0.0005, **kwargs):
        """
        Downloads the tiles as initialized.

        Parameters
        --------------------------------

        getMasks:bool (default= False)
            Download the road PNG mask tile if true

        latLonResolution:float (default= 0.0005)
            The step size to use when creating tiles

        Also accepts kwargs for `get_img`.
        """
        self._getMasks = bool(getMasks)
        min_lat, max_lat = self._getLatBounds()
        min_lon, max_lon = self._getLonBounds()
        if (max_lat - min_lat <= latLonResolution) or (max_lon - min_lon <= latLonResolution):
            # If we fail this check, then our arange will return no
            # results and we'll fetch nothing
            raise ValueError(f"Latitude and longitude bounds must be separated by at least the latLonResolution (currently {latLonResolution}). Either shrink the resolution value or increase the separation of your minimum/maximum latitude and longitude.")

        URL_ALL = []
        for i in tqdm(np.arange(min_lat, max_lat, latLonResolution)):
            tp = None
            for j in np.arange(min_lon, max_lon, latLonResolution):
                URL_ALL.append(self.make_url(i,j))
            if self.verbose:
                print("ALL URL CREATED! ...")
            global LOCK_VAR, UNLOCK_VAR, LOCKING_LIMIT
            if LOCK_VAR == 0:
                if self.verbose:
                    print("LOCKING")
                LOCK_VAR = 1
                UNLOCK_VAR = 0
                tp = ThreadPool(LOCKING_LIMIT)
                tp.imap_unordered(lambda x: self.get_img(x, **kwargs), URL_ALL) #pylint: disable= unnecessary-lambda #cSpell:words imap
                tp.close()
            # SEMAPHORE KINDA THINGIE
            if UNLOCK_VAR >= LOCKING_LIMIT and tp is not None:
                # If we have too many threads running, explicitly call
                # a wait on the threads until the most recent
                # process has cleared. As a practical matter, this will
                # clear _several_ threads and keep up performance
                tp.join()

        



import ssl
import os
import time
import math
import glob
import imghdr
import requests
import sqlite3
import numpy as np
import datetime as dt
from tqdm import tqdm
import multiprocessing
from satellitescraper import map_api
from typing import Tuple
from selenium import webdriver
import chromedriver_autoinstaller
from .file_size import get_folder_size
from multiprocessing.pool import ThreadPool
from os.path import join, exists, normpath, relpath



def generate_summary():
    # Create an approximate analysis of the space required
    total_files_downloaded = cur.execute(''' SELECT * FROM sanity ''')
    total_files_downloaded_val = cur.fetchall() #converts the cursor object to number
    total_number_of_files = len(total_files_downloaded_val)
    print("Total satellite images to be downloaded = ",total_number_of_files)
    print("Total roads tiles to be downloaded = ",total_number_of_files)
    disk_space = 10*2*total_number_of_files/1024
    print("Approx. estimated disk space required = {} MB".format(disk_space))


def create_sanity_db(min_lat_deg, max_lat_deg, min_lon_deg, max_lon_deg, latLonResolution=0.0005, verbose=False):
    # To save all the expected file names to be downloaded
    for i in tqdm(np.arange(min_lat_deg, max_lat_deg, latLonResolution)):
        for j in np.arange(min_lon_deg, max_lon_deg, latLonResolution):
            xTile, yTile = sanity_obj.ret_xy_tiles(i,j)
            # print(xTile," ",yTile)
            # create the primary key for tracking values
            key_id = str(xTile)+"_"+str(yTile)
            # write the query
            query_insert = "INSERT OR IGNORE INTO sanity VALUES ('{}','{}','{}','{}','{}')".format(key_id,xTile, yTile, 0, 0)
            # Insert a row of records
            cur.execute(query_insert)


def update_sanity_db(folder_name):
    # to take the files present in the folder and update all the entries of the 
    # sanity database
    print("Updating sanity db ...")
    all_files_folder = glob.glob('{}/*'.format(folder_name))
    for tile_name in tqdm(all_files_folder):
        if tile_name.count('_') == 1:
            # then it is a satellite imagery
            xTile_val = str(tile_name.split('_')[0]).split('/')[-1]
            yTile_val = str(tile_name.split('_')[-1]).split('.')[0]
            create_id = str(xTile_val)+"_"+str(yTile_val)
            # print(create_id)
            cur.execute('''UPDATE sanity SET satellite_tile = 1 WHERE id = ?''',(str(create_id),))	# set the satellite_tile to 1

        if tile_name.count('_') == 2:
            # then it is a road mask 
            xTile_val = str(tile_name.split('_')[0]).split('/')[-1]
            yTile_val = str(tile_name.split('_')[-2])
            create_id = str(xTile_val)+"_"+str(yTile_val)
            # print(create_id)
            cur.execute('''UPDATE sanity SET road_tile = 1 WHERE id = ?''',(str(create_id),))	# set the road_tile to 1
    con.commit()



def shall_stop():
    # this function returns 1 if we need to stop, i.e., if all the entries are 1
    # which means all the required files are downloaded in the folder specified
    # even if one file is missing, we return 0
    
    # get all the number of 0 entries for satellite imagery
    get_sat_0s = cur.execute(''' SELECT * FROM sanity WHERE satellite_tile = 0 ''')
    get_sat_0s_val = cur.fetchall() #converts the cursor object to number
    total_number_of_sat0s = len(get_sat_0s_val)
    print("Total number of satellite images needed to be downloaded = ", total_number_of_sat0s)

    get_road_0s = cur.execute(''' SELECT * FROM sanity WHERE road_tile = 0 ''')
    get_road_0s_val = cur.fetchall() #converts the cursor object to number
    total_number_of_road0s = len(get_road_0s_val)
    print("Total number of satellite images needed to be downloaded = ", total_number_of_road0s)

    if total_number_of_sat0s == 0 and total_number_of_road0s == 0:
        return 1
    return 0


def check_downloading():
    # checks if the multiprocessing tool is still downloading the files or not
    # if there is a minute increase in byte size of the folder, we need to wait
    # till the multiprocessing thread finishes its execution
    get_folder_size_ini = get_folder_size('myOutputFolder')
    time.sleep(15)
    get_folder_size_final = get_folder_size('myOutputFolder')
    diff = get_folder_size_final - get_folder_size_ini
    speed_download = diff/(15.0*1024*1024) # get the speed in MB
    if diff > 0:
        # we need to sleep for 5 seconds again
        print("Downloading speed == {} MiB/s ".format(speed_download))
        return 1
    return 0


def get_sat_img_id():
    # to get all the satellite image ids which are not yet being downloaded
    get_sat_0s = cur.execute(''' SELECT id FROM sanity WHERE satellite_tile = 0 ''')
    get_sat_0s_val = cur.fetchall() #converts the cursor object to number
    # print("Total number of satellite images needed to be downloaded = ", len(get_sat_0s_val))
    get_sat_ids = []
    for item in get_sat_0s_val:
        get_sat_ids.append(item[0])
    return get_sat_ids
    

def get_road_img_id():
    # to get all the road  tiles image ids which are not yet being downloaded
    get_road_0s = cur.execute(''' SELECT * FROM sanity WHERE road_tile = 0 ''')
    get_road_0s_val = cur.fetchall() #converts the cursor object to number
    # print("Total number of satellite images needed to be downloaded = ", len(get_road_0s_val))
    get_road_ids = []
    for item in get_road_0s_val:
        get_road_ids.append(item[0])
    return get_road_ids




def sanity_check(min_lat_deg, max_lat_deg, min_lon_deg, max_lon_deg, zoom, verbose, threads_, container_dir = "myOutputFolder"):
    # This function contains the main loop for checking the sanity of download
    # till all the files are downloaded

    # Create table sanity with the coordinates, and the corresponding
    # satellite tile and the road tile, id as the primary key xTile_yTile

    cur.execute('''CREATE TABLE IF NOT EXISTS sanity
                (id TEXT primary key, xTile INTEGER, yTile INTEGER, satellite_tile INTEGER, road_tile INTEGER )''')

    # check if the files are downloading or not, if so, then wait for certain seconds,
    # repeat this till the files stop downloading and then start the next batch of downloads
    batch = 1 
    create_sanity_db(min_lat_deg, max_lat_deg, min_lon_deg, max_lon_deg, latLonResolution=0.0005, verbose=False)  
    generate_summary()

    while(shall_stop() == 0):

        sat_img_ids = get_sat_img_id()
        road_img_ids = get_road_img_id()
        # print(sat_img_ids)
        # print(road_img_ids)

        

        while(check_downloading()==1):
            print("Waiting for 15 seconds... Busy downloading")

        print("Batch ============================================================================= ",batch)
        print("===================================================================================")
        batch += 1

        # begin the operation here
        # url_str = [xtile, ytile]
        # TODO

         # To get the maximum number of threads
        MAX_CORES = multiprocessing.cpu_count()
        if threads_> MAX_CORES:
            print("Sorry, {} -- threads unavailable, using maximum CPU threads : {}".format(threads_,MAX_CORES))
            threads_ = MAX_CORES
        
        LOCKING_LIMIT = threads_

        tp=None
        URL_ALL = []
        print("Downloading all the satellite tiles: ")
        for sat_tile_name in sat_img_ids:
            xTile = sat_tile_name.split('_')[0]
            yTile = sat_tile_name.split('_')[1]
            URL_ALL.append([xTile, yTile])

        # print(URL_ALL)
        tp = ThreadPool(LOCKING_LIMIT)
        tp.imap_unordered(lambda x: sanity_obj.get_img(x), URL_ALL) #pylint: disable= unnecessary-lambda #cSpell:words imap
        tp.close()

        # while(tp is not None):
        #     print("Waiting for thread to finish downloading satellite tiles")
        #     time.sleep(5)
        update_sanity_db('myOutputFolder')
        # Save (commit) the changes
        con.commit()

        # continue the loop till there is no file left to download
        # generate the summary
        
        

    # We can also close the connection if we are done with it.
    # Just be sure any changes have been committed or they will be lost.
    print("************************* Download Sucessful *************************")
    con.close()


# connect to the temporary database that we shall use
con = sqlite3.connect('temp_sanity.sqlite')
cur = con.cursor()

# create the object of class longitude's api
sanity_obj = map_api(min_lat_deg = 10,
                    max_lat_deg = 10.01,
                    min_lon_deg = 10,
                    max_lon_deg = 10.01,
                    zoom = 19,
                    verbose = False,
                    threads_ = 5, 
                    container_dir = "myOutputFolder")



if __name__ == "__main__":
    # use main function for proper structuing of code
    sanity_check(min_lat_deg = 10,
                    max_lat_deg = 10.01,
                    min_lon_deg = 10,
                    max_lon_deg = 10.01,
                    zoom = 19,
                    verbose = False,
                    threads_ = 5, 
                    container_dir = "myOutputFolder")


import cv2
import ssl
import os
import time
import math
import glob
import imghdr
import requests
import sqlite3
import numpy as np
import datetime as dt
from tqdm import tqdm
import multiprocessing
from satellitescraper import map_api
from typing import Tuple
from selenium import webdriver
import chromedriver_autoinstaller
from multiprocessing.pool import ThreadPool
from os.path import join, exists, normpath, relpath


def update_stitcher_db(folder_name = "myOutputFolder"):
    # this function stiches the whole folder passed to it
    
    # to take the files present in the folder and update all the entries of the 
    # sticher database

    # Create table sticher with the coordinates, and the corresponding
    # satellite tile and the road tile, id as the primary key xTile_yTile

    cur.execute('''CREATE TABLE IF NOT EXISTS sticher
                (id TEXT primary key, xTile INTEGER, yTile INTEGER, satellite_tile INTEGER, road_tile INTEGER )''')
    
    print("Updating sticher db ...")

    all_files_folder = glob.glob('{}/*'.format(folder_name))
    for tile_name in tqdm(all_files_folder):
        if tile_name.count('_') == 1:
            # then it is a satellite imagery
            xTile_val = str(tile_name.split('_')[0]).split('/')[-1]
            yTile_val = str(tile_name.split('_')[-1]).split('.')[0]
            create_id = str(xTile_val)+"_"+str(yTile_val)
            # print(create_id)
            
            # create the primary key for tracking values
            key_id = str(xTile_val)+"_"+str(yTile_val)
            # write the query
            query_insert = "INSERT OR IGNORE INTO sticher VALUES ('{}','{}','{}','{}','{}')".format(key_id,xTile_val, yTile_val, 0, 0)
            # Insert a row of records
            cur.execute(query_insert)
            cur.execute('''UPDATE sticher SET satellite_tile = 1 WHERE id = ?''',(str(create_id),))	# set the satellite_tile to 1

        if tile_name.count('_') == 2:
            # then it is a road mask 
            xTile_val = str(tile_name.split('_')[0]).split('/')[-1]
            yTile_val = str(tile_name.split('_')[-2])

            # create the primary key for tracking values
            key_id = str(xTile_val)+"_"+str(yTile_val)
            # write the query
            query_insert = "INSERT OR IGNORE INTO sticher VALUES ('{}','{}','{}','{}','{}')".format(key_id,xTile_val, yTile_val, 0, 0)
            # Insert a row of records
            cur.execute(query_insert)

            create_id = str(xTile_val)+"_"+str(yTile_val)
            # print(create_id)
            cur.execute('''UPDATE sticher SET road_tile = 1 WHERE id = ?''',(str(create_id),))	# set the road_tile to 1
    con.commit()

    get_sat_0s = cur.execute(''' SELECT * FROM sticher WHERE satellite_tile = 0 ''')
    get_sat_0s_val = cur.fetchall() #converts the cursor object to number
    total_number_of_sat0s = len(get_sat_0s_val)
    print("Total number of satellite images needed to be downloaded = ", total_number_of_sat0s)

    get_road_0s = cur.execute(''' SELECT * FROM sticher WHERE road_tile = 0 ''')
    get_road_0s_val = cur.fetchall() #converts the cursor object to number
    total_number_of_road0s = len(get_road_0s_val)
    print("Total number of satellite images needed to be downloaded = ", total_number_of_road0s)

    # in case if all the files are not present in the database then we need to exit from the program
    if total_number_of_sat0s > 0 or total_number_of_road0s > 0:
        print("Please download all the tiles properly for this purpose, some tiles may be missing...")
        print("Exiting...")
        exit()



def get_bbox_lat_lon():
    # this function obtains the maximum, minimum latitudes and longitudes present in the database
    print("Calculating bounding boxes for tiles :: ")
    get_all_data = cur.execute(''' SELECT * FROM sticher ''')
    get_all_data_vals = cur.fetchall() #converts the cursor object to number
    total_number_of_data = len(get_all_data_vals)
    print("Total number of rows present in the database= ", total_number_of_data)

    get_lat_list = []
    get_lon_list = []
    for item in tqdm(get_all_data_vals):
        get_lat_list.append(item[1])
        get_lon_list.append(item[2])
    
    min_lat = min(get_lat_list)
    max_lat = max(get_lat_list)

    min_lon = min(get_lon_list)
    max_lon = max(get_lon_list)

    print("Min lat tile = {}, Max lat tile = {}, Min lon tile = {}, Max lon tile = {}".format(min_lat, max_lat, min_lon, max_lon))

    return [min_lat, max_lat, min_lon, max_lon]


def stitch_whole_tile(save_name="full_tile", folder_name="myOutputFolder"):
    # This function stitches the whole tile and saves in <save_name>_sat.png and <save_name>_road.png
    # firstly update the stitcher databse
    update_stitcher_db(folder_name="myOutputFolder")

    # get the bounding box
    min_lat, max_lat, min_lon, max_lon = get_bbox_lat_lon()

    diff_lat = max_lat - min_lat
    diff_lon = max_lon - min_lon

    print("No. of tiles in latitude = {}, and longitude = {}".format(diff_lat, diff_lon))

    print("Creating an image of size : {}x{} pixels ...".format(diff_lat*256,diff_lon*256))

    # stich the satellite imagery tile

    SAT_TILE_FULL = np.zeros([diff_lon*256,diff_lat*256,3])

    for lat in tqdm(range(min_lat, max_lat)):
        for lon in range(min_lon, max_lon):
            # stich the tiles together
            img_name = folder_name+"/"+str(lat)+"_"+str(lon)+".jpg"
            sat_img = cv2.imread(img_name)
            # print(sat_img.shape)
            x_1 = int(256*(lat-min_lat+1))-256
            x_2 = int(256*(lat-min_lat+1))

            y_1 = int(256*(lon-min_lon+1)-256)
            y_2 = int(256*(lon-min_lon+1))
            # print(x_1," ",x_2," ",y_1," ",y_2)
            SAT_TILE_FULL[y_1:y_2, x_1:x_2] = sat_img
    
    sat_save_name = save_name+"_sat.png"
    cv2.imwrite(sat_save_name,SAT_TILE_FULL)

    # stich the road imagery tile

    ROAD_TILE_FULL = np.zeros([diff_lon*256,diff_lat*256,3])

    for lat in tqdm(range(min_lat, max_lat)):
        for lon in range(min_lon, max_lon):
            # stich the tiles together
            img_name = folder_name+"/"+str(lat)+"_"+str(lon)+"_road.png"
            sat_img = cv2.imread(img_name)
            # print(sat_img.shape)
            x_1 = int(256*(lat-min_lat+1))-256
            x_2 = int(256*(lat-min_lat+1))

            y_1 = int(256*(lon-min_lon+1)-256)
            y_2 = int(256*(lon-min_lon+1))
            # print(x_1," ",x_2," ",y_1," ",y_2)
            ROAD_TILE_FULL[y_1:y_2, x_1:x_2] = sat_img
    
    sat_save_name = save_name+"_road.png"
    cv2.imwrite(sat_save_name,ROAD_TILE_FULL)







# connect to the temporary database that we shall use
con = sqlite3.connect('sticher.sqlite')
cur = con.cursor()

if __name__ == "__main__":
    # use main function for proper structuing of code
    update_stitcher_db("myOutputFolder")


__name__ = "satellitescraper"
__version__ = "0.1.3"
__author__ = "Mostafa Mabrok| mostafa.m.mabrok@gmail.com"
__release_date__ = '19-July-2024'
from .satellitescraper import map_api
from .file_size import get_folder_size
from .sanity_checker import sanity_check
from .tiles_sticher import update_stitcher_db, get_bbox_lat_lon, stitch_whole_tile



import setuptools
import re
import ast

setuptools.setup(
    name="satellitescraper",
    version="0.1.3",
    author="Mostafa Mabrok",
    author_email="mostafa.m.mabrok@gmail.com",
    description="Python Package for Scraping Sattelite Image Data",
    long_description="PLACEHOLDER",
    long_description_content_type="text/markdown",
    url="https://github.com/Mostafamabrok/longitude",
    install_requires=['certifi','chardet','chromedriver-autoinstaller','idna',
                      'numpy','requests','selenium','tqdm','urllib3', 
                      'opencv-python'],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)

